/* 1.3  10-08-85						(ftoa.c)
 ************************************************************************
 *			Robert C. Tausworthe				*
 *			Jet Propulsion Laboratory			*
 *			Pasadena, CA 91009		1985		*
 *			rewritten to modern C by Jiri Pittner           *
 *			jiri@pittnerovi.com
 ************************************************************************/

typedef unsigned char BOOL;

#define DPRECISION 8
#define BIG10X 6
#define SYMLEAST (.1-1e-7)

#include <string.h>
#include <stdio.h>
#include <ctype.h>


/************************************************************************/
char *itoab(char *s, int n, int w, int b)	/* convert n to a minimum of |w| ascii characters,
			   base b, and put  in s.  Return pointer to s.
			   0-fill if w<0, else space-fill.		*/
{
	char digits[] = "0123456789abcdefghijklmnopqrstuvwxyz";
	int i, j;
	BOOL sign;
	char c, fill;

	if (n < 0)
	{	n = -n;
		sign = 1;
	}
	else
		sign = 0;
	if (w < 0)
	{	fill = '0';
		w = -w;
	}
	else
		fill = ' ';
	i = 0;
	do			/* generate digits in reverse order:	*/
	{	s[i++] = digits[n % b];
	} while (n /= b);
	if (sign)			/* put the sign into the string.*/
		s[j = i++] = '-';
	else
		j = i - 1;
	while (i < w)			/* expand to proper width.	*/
		s[i++] = fill;	/* i is just beyond last filled postion	*/
	if (--i != j && sign && fill == '0')
	{	s[i] = s[j];		/* get the sign.		*/
		s[j] = fill;		/* fill in over it		*/
	}
	s[++i] = 0;			/* add null terminator	*/
	for (i--, j = 0; j < i; j++, i--) /*  now reverse chars */
	{	c = s[j];
		s[j] = s[i];
		s[i] = c;
	}
	return s;
}

/************************************************************************/
char *itoa(char *s, int n, int w)	/* return pointer to s, which contains ascii value of
		   n to width |w|, 0-filled if w<0, else space-filled.	*/
{
	return itoab(s, n, w, 10);
}
/*----------------------------------------------------------------------*/

float roundoff(float x, int *d, int p)	/* return x * 10^*d rounded to precision p	*/
{				   /*************************************/
	 float rounder[21] = /* NOTE: DPRECISION must be no larger*/
				   /* than the number of elements in	*/
				   /* this array.			*/
				   /*************************************/
	{	0.5,     0.5e-1,  0.5e-2,  0.5e-3,  0.5e-4,  0.5e-5,
		0.5e-6,  0.5e-7,  0.5e-8,  0.5e-9,  0.5e-10, 0.5e-11,
		0.5e-12, 0.5e-13, 0.5e-14, 0.5e-15, 0.5e-16, 0.5e-17,
		0.5e-18, 0.5e-19, 0.5e-20
	};
	float y;

	if ((y = x) != 0.0)
		if (p <= DPRECISION)
			if ((x += rounder[p]) >= 10.0)
			{	x = y / 10.0 + rounder[p];
				++*d;
			}
	return x;
}

int pickdigit(float *x)	/* return ascii leading digit of x, strip it off, and
		   prepare x for extraction of next digit.		*/
{
	int n;

	n = *x;			/* take integer part.			*/
	*x = 10 * ( *x - n);	/* strip it off, and renormalize x	*/
	return (n + '0');
}



/*----------------------------------------------------------------------*/
void floatform(char *s, float x, int d, int pr)	/* put x * 10^d (precision pr) into string s
			   in f-format.					*/
{
	int i;

	x = roundoff(x, &d, pr + ((d > 0) ? d : 0));
	if (d < 0)
		*s++ = '0';
	else
		while (d >= 0)
		{	*s++ = pickdigit(&x);
			d--;
		}
	if ((i = pr) >= 0)
		*s++ = '.';
	while (++d < 0 && i-- > 0)
		*s++ = '0';
	while (i-- > 0)
		*s++ = pickdigit(&x);
	*s = 0;
}

/*----------------------------------------------------------------------*/
void expform(char *s, float x, int d, int pr)	/* put x * 10^d (precision pr) into string s in e-format					*/
{
	int i;

	x = roundoff(x, &d, i = pr);
	*s++ = pickdigit(&x);
	*s++ = '.';
	while (i-- > 0)
		*s++ = pickdigit(&x);
	*s++ = 'e';
	itoa(s, d, 0);	/* minimum width */
}

/*----------------------------------------------------------------------*/
void minform(char *s, float x, int d, int pr)	/* put x * 10^d into string s in minimum format
			   with precision pr				*/
{
	int i, j;

	if (d < 5 && d > -5)
	{	floatform(s, x, d, pr);
		i = strlen(s) - 1;
		while (s[i] == '0')
			s[i--] = 0;
		if (s[i] == '.')
			s[i--] = 0;
	}
	else
	{	expform(s, x, d, pr);
		j = strlen(s) - 1;
		while (s[j] != 'e')
			j--;
		for (i = j - 1; s[i] == '0'; i--)
			;
		strcpy(&s[++i], &s[j]);
	}
}

/************************************************************************/
	
float normalize(float x, int *d)		/* return x * 10^-d in [1, 10), with d set so this condition is met.			*/
{
	/****************************************************************
	 * This array contains 2^n powers of 10 up to n = BIG10X-1	*
	 ****************************************************************/

	 float tentothe2tothe[BIG10X] =
	{	10., 100., 10000., 100000000., 10000000000000000.,
		100000000000000000000000000000000.
	};

	/********************************************************
	 * Also, make this array be 2^n up to 2^(BIG10X-1)	*
	 ********************************************************/

	 int twoto[BIG10X] =
	{	1, 2, 4, 8, 16, 32
	};

	int i;

	*d = 0;
	if (x != 0.0)
   	{
		if (x < 1.0)
		{	
			for (i = 1; x < SYMLEAST; i++) x *= 10.0;	
			x = 10.0 / normalize(1.0 / x, d);
			*d = -*d - i;
			if (x >= 10.0)	/* compensate if on the edge.	*/
			{	x /= 10.0;
				++*d;
			}
		}	
		else
			for (i = BIG10X; x >= 10.0 && --i >= 0; )
				if (x >= tentothe2tothe[i])
				{	x /= tentothe2tothe[i];
					*d += twoto[i];
				}
	}
	return x;
}	


/************************************************************************/
char *ftoa0(char *s, float x, int pr, int mode)	/* Convert x to ascii string s with precision pr
			   in mode 'e', 'f', or 'g' format.
			   Return pointer to s.				*/
{
	BOOL minus;
	char *p;
	int d;

	p = s;		/* save string pointer for returns	*/
	*s = 0;
	if (pr > DPRECISION) pr = DPRECISION;
	if ((minus = (x < 0.0))) /* work with pos. numbers */
		x = -x;
	x = normalize(x, &d);
	if(minus) *s++ = '-'; //(minus ? '-' : ' ');	
	switch (tolower(mode))
	{	
		case 'e':
		case 0:
			expform(s, x, d, pr);
			break;
		case 'f':
		case 1:
			floatform(s, x, d, pr);
			break;
		case 'g':
		case 2:
			minform(s, x, d, pr);
			break;
		default:
			;
	}
	return p;
}

char *ftoa(char *s, float x, int len, int mode, int pr)
{
ftoa0(s,x,pr,mode); 
if(len==0) {return s;}
int l0=strlen(s);
if(len<0)
	{
	if(l0<len)
		{
		s+=l0;
		len-=l0;
		while(l0--) *s++ = ' ';
		*s=0;
		}
	return s;
	}
//if len>0
if(l0<len)
	{
	memmove(s+len-l0,s,l0+1);
	memset(s,' ',len-l0);
	} 
return s;
}

/*

int main()
{
char buf[100];
float f;
while(1)
	{
	scanf("%f",&f);
	ftoa(buf,f,-10,'f',5);
	printf("%s\n",buf);
	}
}

*/
